module DEIS
  class Deis < Thor
    include Thor::Shell
    class_option :verbose, :type => :boolean, :default => false
    class_option :environment , :type => :string, :default => "staging", :aliases => "-e"

    desc "config <export|import|list|set|unset> <varstring>", "set or unset a variable"
    long_desc <<-LONGDESC
    Modifies the the LOCAL configuration variables in the current environment (set with --environment or -e). When deployed, these will become environment variables for your application.

    Examples:

    `rdeis config list`
    `rdeis config`

    Returns all config items for this environment. List is the default action for config.

    `redis export $destination_file -e $environment`

    Export all data for the set $environment into the $file path specified. This will be in a JSON format

    `redis import $origin_file -e $environment`

    Import and replace current setup for $environment with the data contained in $origin_file

    `rdeis config list TEST`

    Filters the list to just the value for TEST (if present)

    `rdeis config set FOO=BAR TEST=1`

    This sets two variables, FOO & TEST, to BAR & 1 respectively and matches the syntax for the `deis` binary

    `rdeis config unset FOO TEST`

    This will unset two variables, FOO & TEST, as long as they exist in the configuration settings.

    You will need to deploy for changes to effect the running environment.

    LONGDESC
    def config(action="list", *vars)
      self.bootstrap
      if DEIS::Config.method_defined? action
        config = DEIS::Config.new(options)
        config.send(action, vars)
      else
        say "invalid option", :red
      end
    end



    desc "domains <add|remove|list> <domain>", "add, remove or list domains"
    long_desc <<-LONGDESC
    Modifies the the LOCAL domains in the current environment (set with --environment or -e). Once deployed, this will be the domains that route to your application.

    Examples:

    `rdeis domains list`
    `rdeis domains`

    Returns all domains for this environment. List is the default action.

    `rdeis domains add test.example.com foo.bar.com`

    This adds two domains to the setup for this application.

    `rdeis domains remove foo.bar.com`

    This will unset foo.bar.com, as long as they exist in the settings.

    You will need to deploy for changes to effect the running environment.


    LONGDESC
    def domains(action="list", *domains)
      self.bootstrap
      if DEIS::Domains.method_defined? action
        domain = DEIS::Domains.new(options)
        domain.send(action, domains)
      else
        say "invalid option", :red
      end
    end

    desc "deploy <all|create|config|domains|gems|git|remotes>", "deploy this repo"
    option :proxy, :type => :string, :aliases => "-P", :banner => "<Hostname|IP> of proxy host, for vpc etc"
    option :cluster, :type => :string, :default => "staging", :banner => "<name> of the cluster to deploy to"
    option :path, :type => :string, :aliases => "-p", :banner => "<FILE_PATH> to deploy to"
    long_desc <<-LONGDESC
    Deploy to the deis cluster that is configured. This will run over the actions to checkout the git repo to a set location, create the deis application, use your local config and domain values on the deis application, find and add the deis remote to local version and the run a push to that remote

    Examples

    `rdeis deploy`
    `rdeis deploy all`

    This will run all elements of the checkout

    `rdeis create`

    Runs just a deis create $application --cluster=$cluster

    `rdeis config`

    Deploys the local config to the deis application

    `rdeis domains`

    Deploys the local domain list to the deis application

    `rdeis gems`

    Modifies your local gem file and convert and git@github based gems to run from oauth basic token access from an ENV variable

    `rdeis git`

    Create and clone the repo at --path

    `rdeis remotes`

    Find and setup deis remotes on current directory

    LONGDESC
    def deploy(action="all")
      self.bootstrap
      if DEIS::Deploy.method_defined? action
        deploy = DEIS::Deploy.new(options)
        deploy.send(action)
      else
        say "invalid option", :red
      end
    end

    desc "scale <varstring>", "Scale a proc"
    long_desc <<-LONGDESC
    Scale a proc up or down based on value you enter.

    Examples:

    `rdeis scale web=1`
    Would turn on 1 instance of the web proc

    LONGDESC
    def scale(*vars)
      self.bootstrap
      if DEIS::Deploy.method_defined? "scale"
        deploy = DEIS::Deploy.new(options)
        deploy.send("scale", vars)
      else
        say "invalid option", :red
      end
    end

    desc "bootstrap <type>", "first run install setup funcs"
    def bootstrap(action="all")
      self.updgrade
      if DEIS::Bootstrap.method_defined? action
        bootstrap = DEIS::Bootstrap.new(options)
        bootstrap.send(action)
      else
        say "invalid option", :red
      end
    end

    desc "updgrade <from>", "See if older version exists we can copy over"
    def upgrade(from=nil)
      run = DEIS::Upgrade.new(options, from)
      run.all
    end

    desc "version", "See version number"
    def version
      say "rdeis #{DEIS::VERSION}"
    end

    desc "destroy", "Destroy the application"
    def destroy
      run = DEIS::Destroy.new(options)
      run.all
    end

  end
end